*==============================================================================
* Program:				MRUDRV1.PRG
* Purpose:				Pre-generation driver for MRU menus.
* 						Modifies MNX, adding code in Setup, Cleanup
*                       and to Command field of each bar.
* Author:				Tamar E. Granor
* Copyright:			(c) 2004 Tamar E. Granor, may be distributed freely
*                       as long as this copyright notice remains.
* Last revision:		05/13/04
* Parameters:			None
* Returns:				None
* Environment in:		Called from GenMenuX MNXDRV1 hook.
* Environment out:		Menu table has been changed.
*==============================================================================

#DEFINE CRLF CHR(13) + CHR(10)

LOCAL nCommentLines, aCommLines[1], nLine
LOCAL cSetupCode, cOnExitCode, cCleanupCode
LOCAL lAlwaysShow, lExitOption
PUBLIC jcMRUTable, jcMRUDayVar

gcExpand = ""
gcRedraw = ""

* Extract name for MRU tracking table from Comment field
* of first record
IF "*:MRUTABLE"$UPPER(Setup)
	* Find it
	nCommentLines = ALINES(aCommLines, Setup)
	nLine = ASCAN(aCommLines, "*:MRUTable",-1, -1, 1,13)
	jcMRUTable = ALLTRIM(STRTRAN(aCommLines[nLine], "*:MRUTable", ""))
ELSE 
	jcMRUTable = "MenuUse"
ENDIF 

* Extract name of variable containing time limit for showing
* bars
IF "*:MRUDAYS"$UPPER(Setup)
	nCommentLines = ALINES(aCommLines, Setup)
	nLine = ASCAN(aCommLines, "*:MRUDays",-1, -1, 1,13)
	jcMRUDayVar = ALLTRIM(STRTRAN(aCommLines[nLine], "*:MRUDays", ""))
ELSE
	jcMRUDayVar = "gnMenuDays"
ENDIF


* Generate code to open menu MRU table
TEXT TO cSetupCode TEXTMERGE NOSHOW 
* Begin insertions from GENMENUX MRU driver

* Declare variable to indicate menu exit
PUBLIC gMRUExiting
gMRUExiting = .F.

* Open menu MRU table; create if 
IF NOT USED("__MRUTable")
	IF NOT FILE("<<jcMRUTable>>")
		CREATE TABLE ("<<jcMRUTable>>") ;
			(iID I AUTOINC, cPopup C(15), cBar C(50), lAlways L, lExit L, tLastUsed T) 
		INDEX ON UPPER(cPopup + cBar) TAG PopupBar
		USE 
	ENDIF
	
	USE ("<<jcMRUTable>>") IN 0 SHARED ALIAS __MRUTable
ENDIF

* Add menu bars to MRU table, if necessary

ENDTEXT 

* Generate code to add bars to MRU table
* Also, if this is an exit bar, generate code to set exit flag

cLevel = ""
SCAN FOR ObjType=3 AND UPPER(LevelName) <> "_MSYSMENU"
	cPrompt = STRTRAN(Prompt, "\<","")
	lAlwaysShow = "*:MRUALWAYS"$UPPER(Comment)
	lExitOption = "*:MRUEXIT"$UPPER(Comment)
	
	TEXT TO cSetupCode ADDITIVE TEXTMERGE NOSHOW 
IF NOT SEEK(UPPER(PADR("<<LevelName>>", LEN(__MRUTable.cPopup)) + "<<cPrompt>>"), "__MRUTable", "PopupBar")
	INSERT INTO ("<<jcMRUTable>>") ;
	    (cPopup, cBar, lAlways, lExit) ;
		VALUES ("<<LevelName>>", "<<cPrompt>>", <<m.lAlwaysShow>>, <<m.lExitOption>>)
ENDIF


	ENDTEXT 
	
	IF lExitOption

		TEXT TO cExitCode NOSHOW 
gMRUExiting = .T.
USE IN SELECT("__MRUTable")

		ENDTEXT
		
		DO CASE 
		CASE NOT EMPTY(Command)
			REPLACE Command WITH Command + CRLF + cExitCode
			
		CASE NOT EMPTY(Procedure)
			* Add at beginning to avoid getting cut off
			* by RETURN
			REPLACE Procedure WITH cExitCode + CRLF + Procedure
			
		OTHERWISE
			* Nothing to do
		ENDCASE
	ENDIF 

ENDSCAN

GO top

REPLACE Setup WITH Setup + CRLF + cSetupCode


* Add ON EXIT POPUP code to clean-up code once for each popup
cOnExitCode = ""
SCAN FOR ObjType = 2 AND UPPER(Name) <> "_MSYSMENU"

	TEXT TO cOnExitCode TEXTMERGE NOSHOW ADDITIVE 
ON EXIT POPUP <<ALLTRIM(LevelName)>> ;
   DO UpdateMenu WITH POPUP(),BAR() ;
   IN ("<<jcOutFile>>")

	ENDTEXT 

ENDSCAN 

GO TOP

* Add code to store menu usage to Cleanup code
TEXT TO cCleanupCode TEXTMERGE NOSHOW 

PROCEDURE UpdateMenu
* Update menu usage table when item is chosen
LPARAMETERS cPopup, nBar

LOCAL cBarPrompt

cBarPrompt = STRTRAN(PRMBAR(ALLTRIM(m.cPopup), m.nBar),"<\","")

UPDATE ("<<jcMRUTable>>") ;
	SET tLastUsed = DATETIME() ;
	WHERE UPPER(cPopup) == UPPER(ALLTRIM(m.cPopup)) ;
	  AND UPPER(cBar) == UPPER(ALLTRIM(m.cBarPrompt))
	  
RETURN

ENDTEXT 

REPLACE Cleanup WITH cOnExitCode + CRLF + Cleanup + CRLF + cCleanupCode

* Set up initial redrawing of bars
cCleanUpCode = ""
SCAN FOR ObjType=2 AND ObjCode = 0

TEXT TO cCleanUpCode ADDITIVE TEXTMERGE NOSHOW 
DO Redraw WITH "<<Name>>" IN ("<<jcOutFile>>")

ENDTEXT 

ENDSCAN 

GO TOP 

REPLACE Cleanup WITH cCleanUpCode + CRLF + Cleanup

RETURN
