*==============================================================================
* Program:				MRUDRV6.PRG
* Purpose:				Post-generation driver for MRU menus.
* 						Takes generated code and modifies it to support
* 						MRU adaptive menus.
* Author:				Tamar E. Granor
* Copyright:			(c) 2004 Tamar E. Granor, may be distributed freely
*                       as long as this copyright notice remains.
* Last revision:		05/13/04
* Parameters:			None
* Returns:				None
* Environment in:		Called from GenMenuX MPRDRV1 hook.
* Environment out:		Generated code has been changed.
*==============================================================================

#DEFINE CRLF CHR(13) + CHR(10)
#DEFINE TAB CHR(9)

LOCAL cMPR, nStartDefn, nEndDefn, cStartDefn
LOCAL aCodeLines[1], nLineCount
LOCAL aBars[1,2], nBars, nBar, nCurrentBar, lInPostCode
LOCAL cRedrawCode, cExpandCode
PRIVATE nHighBar, cPopup

* Grab generated code
cMPR = Object

* Find the part of the code that defines popups and bars
nLineCount = ALINES(aCodeLines, cMPR)
cStartDefn = "DEFINE POPUP"
nStartDefn = ASCAN(aCodeLines, cStartDefn)
IF nStartDefn = 0
	* Nothing to do
	RETURN
ENDIF

* Extract the popup and bar definitions
nEndDefn = ASCAN(aCodeLines, ;
                 "*" + REPLICATE(" ", 7) + REPLICATE("*", 57), ;
                 nStartDefn)
nEndDefn = nEndDefn - 1 

* Build RedrawMenu code
cRedrawCode = ""
cExpandCode = ""
nBars = 0
nHighBar = 0

* Set up release of bars 
TEXT TO cRedrawCode ADDITIVE TEXTMERGE NOSHOW 

FOR nBar = BARCOUNT(m.tcPopup) TO 1 STEP -1
	RELEASE BAR GETBAR(m.tcPopup,nBar) OF &tcPopup
ENDFOR

DO CASE 

ENDTEXT 

TEXT TO cExpandCode ADDITIVE TEXTMERGE NOSHOW 
FOR nBar = BARCOUNT(m.tcPopup) TO 1 STEP -1
	RELEASE BAR GETBAR(m.tcPopup,nBar) OF &tcPopup
ENDFOR

DO CASE 

ENDTEXT 

FOR nLine = nStartDefn TO nEndDefn
	* Set up case statement for popups
	DO CASE 
	CASE "DEFINE POPUP"$aCodeLines[nLine]
		* Start of new popup. Add code for
		* previous popup to accumulating code.
		* For expansion code, just set up case.
		cRedrawCode = cRedrawCode + BuildDrawBars(@aBars, nBars)
		cExpandCode = cExpandCode + BuildExpandBars(@aBars, nBars)
		
		* Add this DEFINE POPUP
		cPopup = UPPER(STREXTRACT(aCodeLines[nLine], "DEFINE POPUP ", " MARGIN"))
		cCase = "CASE UPPER(m.tcPopup) = '" + cPopup + "'" + CRLF
		
		cRedrawCode = cRedrawCode + cCase
		cExpandCode = cExpandCode + cCase
		
		nCurIndent = 1 
					
		* Now loop through this popup, collecting
		* info for each bar
		nBars = 0
		DIMENSION aBars[1,2]
		lInPostCode = .F.
		nHighBar = 0
		
	CASE "DEFINE BAR"$aCodeLines[nLine]
		* Add a bar
		nBars = nBars + 1 
		DIMENSION aBars[nBars,2]
		aBars[nBars,1] = CRLF + TAB + aCodeLines[nLine]
		
		* Keep track of highest bar number used
		nBarNo = VAL(STREXTRACT(aCodeLines[nLine]," BAR ", " OF "))
		aBars[nBars,2] = nBarNo
		nHighBar = MAX(nHighBar, nBarNo)
		
	CASE "ON SELECTION"$aCodeLines[nLine]
		* Add to appropriate bar
		IF NOT lInPostCode
			lInPostCode = .t.
		ENDIF
		nBarNo = VAL(STREXTRACT(aCodeLines[nLine], " BAR ", " OF "))
		nCurrentBar = ASCAN(aBars, nBarNo, -1, -1, 2, 8)
		
		aBars[nCurrentBar,1] = aBars[nCurrentBar,1] + CRLF + ;
		                     TAB  + aCodeLines[nLine]
	
	CASE "ON BAR"$aCodeLines[nLine]
		* Add to appropriate bar
		IF NOT lInPostCode
			lInPostCode = .t.
		ENDIF
		nBarNo = VAL(STREXTRACT(aCodeLines[nLine], " BAR ", " OF "))
		nCurrentBar = ASCAN(aBars, nBarNo, -1, -1, 2, 8)

		aBars[nCurrentBar,1] = aBars[nCurrentBar,1] + CRLF + ;
		                     TAB + aCodeLines[nLine]
	
	CASE "ON EXIT"$aCodeLines[nLine]
		* Add to appropriate bar
		aBars[nCurrentBar,1] = aBars[nCurrentBar,1] + CRLF + ;
		                     TAB  + aCodeLines[nLine]
	
	CASE NOT lInPostCode 
		* This is part of DEFINE BAR statement.
		* Add to bar under construction
		aBars[nBars,1] = aBars[nBars,1] + CRLF + ;
		               + TAB + aCodeLines[nLine]

	CASE lInPostCode
		* This is part of ON EXIT or ON SELECTION BAR statement.
		* Add to bar under construction
		aBars[nCurrentBar,1] = aBars[nCurrentBar,1] + CRLF + ;
 		                     + TAB + aCodeLines[nLine]
	
	ENDCASE
ENDFOR 

* Add last set of bars
cRedrawCode = cRedrawCode + BuildDrawBars(@aBars, nBars)
cRedrawCode = cRedrawCode + CRLF + "ENDCASE" + CRLF + CRLF

cExpandCode = cExpandCode + BuildExpandBars(@aBars, nBars)
cExpandCode = cExpandCode + CRLF + "ENDCASE" + CRLF + CRLF

* Generate redraw code
REPLACE Object WITH Object + CRLF + ;
   "PROCEDURE Redraw" + CRLF + ;
   "LPARAMETER tcPopup" + CRLF + CRLF + ;
   "LOCAL lAnySkipped" + CRLF + CRLF + ;
   "lAnySkipped = .F." + CRLF + CRLF + ;
   "m.tcPopup = UPPER(m.tcPopup)" + CRLF + CRLF + ;
   cRedrawCode + CRLF + ;
   "RETURN"

* Generate expand code
REPLACE Object WITH Object + CRLF + ;
   "PROCEDURE ExpandMenu" + CRLF + ;
   "LPARAMETER tcPopup" + CRLF + CRLF + ;
   "LOCAL nBar, cInvert" + CRLF + CRLF + ;
   "m.tcPopup = UPPER(m.tcPopup)" + CRLF + CRLF + ;
   cExpandCode + CRLF + ;
   "RETURN"
   
* Get rid of public variables
RELEASE jcMRUTable, jcMRUDayVar

RETURN    

PROCEDURE BuildDrawBars
* Build the redraw code for a set of bars
LPARAMETERS aBars, nBars

LOCAL nBar, cResult, aTabbedBars[1]

IF nBars = 0
	RETURN ""
ENDIF

* Copy bars and add a tab in front of each line
DIMENSION aTabbedBars[nBars,2]
ACOPY(aBars, aTabbedBars)
FOR nBar = 1 TO nBars
	aTabbedBars[ nBar,1 ] = TAB + STRTRAN(aTabbedBars[ nBar,1], CHR(10), CHR(10) + TAB)
ENDFOR

cResult = ""
FOR nBar = 1 TO nBars
	* Get item name and popup name
	cPopup = STREXTRACT(aTabbedBars[nBar,1], " OF ", " PROMPT")	
	cPrompt = STREXTRACT(aTabbedBars[nBar,1], ' PROMPT "', '"')
	cPrompt = STRTRAN(cPrompt, "\<","")

	TEXT TO cResult ADDITIVE TEXTMERGE NOSHOW 

	SEEK UPPER(PADR("<<cPopup>>",LEN(cPopup)) + "<<cPrompt>>") IN __MRUTable ORDER PopupBar
	IF NOT FOUND("__MRUTable") OR (__MRUTable.lAlways OR __MRUTable.tLastUsed >= DATE()-m.<<jcMRUDayVar>>)
		<<aTabbedBars[nBar,1]>>
	ELSE
		lAnySkipped = .t.
	ENDIF
	

	ENDTEXT 
ENDFOR 	

TEXT TO cResult ADDITIVE TEXTMERGE NOSHOW 
	IF lAnySkipped
		DEFINE BAR <<INT(nHighBar + 1)>> OF <<cPopup>> PROMPT "MRU item" ;
			MRU
		ON SELECTION BAR <<INT(nHighBar + 1)>> OF <<cPopup>> ;
	   		DO ExpandMenu WITH "<<cPopup>>" in ("<<jcOutFile>>")
ENDIF


ENDTEXT 

RETURN cResult

PROCEDURE BuildExpandBars
* Build bar definition code for expanding menu
LPARAMETERS aBars, nBars

LOCAL nBar, cResult
LOCAL nBarLines, nLine, aBarLines[1], lFoundDefine
LOCAL lFoundDefineEnd, cBarCode

IF nBars = 0
	RETURN ""
ENDIF

cResult = ""
FOR nBar = 1 TO nBars
	* Get item name and popup name
	cPopup = STREXTRACT(aBars[nBar,1], " OF ", " PROMPT")	
	cPrompt = STREXTRACT(aBars[nBar,1], ' PROMPT "', '"')
	cPrompt = STRTRAN(cPrompt, "\<","")

	* Find the last line of the DEFINE BAR statement and add invert code
	nBarLines = ALINES(aBarLines, aBars[nBar,1])
	lFoundDefine = .F.
	lFoundDefineEnd = .F.
	cBarCode = ""
	FOR nLine = 1 TO nBarLines
		cBarCode = cBarCode + aBarLines[nLine]
		IF NOT lFoundDefineEnd
			* Once code is added, no further processing is needed
			IF "DEFINE BAR"$aBarLines[nLine]
				lFoundDefine = .T.
			ENDIF
			IF lFoundDefine 
				IF RIGHT(ALLTRIM(aBarLines[nLine]),1) <> ";"
					cBarCode = cBarCode + " &cInvert " 
					lFoundDefineEnd = .T.
				ENDIF
			ENDIF
		ENDIF 
		cBarCode = cBarCode + CRLF
	ENDFOR 
	
	TEXT TO cResult ADDITIVE TEXTMERGE NOSHOW 

	SEEK UPPER(PADR("<<cPopup>>",LEN(cPopup)) + "<<cPrompt>>") IN __MRUTable ORDER PopupBar
	IF NOT FOUND("__MRUTable") OR __MRUTable.lAlways OR (NOT EMPTY(__MRUTable.tLastUsed) AND __MRUTable.tLastUsed >= DATE()-m.<<jcMRUDayVar>>)
		cInvert = ""
	ELSE
		cInvert = "INVERT"
	ENDIF
<<cBarCode>>

	ENDTEXT 
ENDFOR 	

TEXT TO cResult ADDITIVE TEXTMERGE NOSHOW 
	ACTIVATE POPUP <<cPopup>>

	IF NOT gMRUExiting
		DO Redraw WITH m.tcPopup IN ("<<jcOutfile>>")
	ENDIF


ENDTEXT 

RETURN cResult